//--------------------------- BASIC PROPERTIES ------------------------------

#include "include\\Warnings.fxh"

float4x4 World;
float4x4 View;
float4x4 Projection;
float4x4 WorldInverseTranspose;

bool SelfShadows;
bool ConstantLight;

float3 DiffuseLightDirection;
float4 DiffuseColor = float4(1, 1, 1, 1);
float DiffuseIntensity = 1.0;
float4 LineColor = float4(0, 0, 0, 1);
float LineThickness;

texture Texture;
 
// The texture sampler, which will get the texture color
sampler2D textureSampler = sampler_state
{
    Texture = (Texture);
    Filter = COMPARISON_MIN_MAG_LINEAR_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};
 
//--------------------------- DATA STRUCTURES ------------------------------
struct VS_INPUT
{
    float4 Position : POSITION0;            // The position of the vertex
    float3 Normal : NORMAL0;                // The vertex's normal
    float2 TextureCoordinate : TEXCOORD0;    // The texture coordinate of the vertex
};
struct VS_OUTPUT
{
    float4 Position : POSITION0;
    float2 TextureCoordinate : TEXCOORD0;
    float3 Normal : TEXCOORD1;
};
 
//--------------------------- SHADERS ------------------------------
// The vertex shader that does cel shading.
// It really only does the basic transformation of the vertex location,
// and normal, and copies the texture coordinate over.
VS_OUTPUT CelVertexShader(VS_INPUT input)
{
    VS_OUTPUT output;
 
    // Transform the position
    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
 
    // Transform the normal
    output.Normal = normalize(input.Normal);
 
    // Copy over the texture coordinate
    output.TextureCoordinate = input.TextureCoordinate;
 
    return output;
}
 
// The pixel shader that does cel shading.  Basically, it calculates
// the color like is should, and then it discretizes the color into
// one of four colors.
float4 CelPixelShader(VS_OUTPUT input) : COLOR0
{
	float4 normal;
	if(ConstantLight)
		normal = mul(mul(mul(input.Normal, World), View), Projection);
	else
		normal = mul(input.Normal, World);
	normal = normalize(normal);
	normal.w = 1;
    // Calculate diffuse light amount
    float intensity = dot(normalize(DiffuseLightDirection), normal);
    if(intensity < 0)
        intensity = 0;
	if(intensity > 1)
        intensity = 1;
 
    // Calculate what would normally be the final color, including texturing and diffuse lighting
    float4 color = tex2D(textureSampler, input.TextureCoordinate) * DiffuseColor * DiffuseIntensity;
    color.a = 1;
 
	//return color;
    // Discretize the intensity, based on a few cutoff points
	if(SelfShadows)
	{
		if (intensity > 0.35)
        color = float4(1,1,1,1) * color;
		/*else if (intensity > 0.2)
			color = float4(0.7,0.7,0.7,1) * color;*/
		else //if (intensity > 0.05)
			color = float4(0.65,0.65,0.65,1) * color;
		/*else
			color = float4(0.1,0.1,0.1,1) * color;*/
	}
	else
		color = float4(1,1,1,1) * color;

    return color;
}
 
// The vertex shader that does the outlines
VS_OUTPUT OutlineVertexShader(VS_INPUT input)
{
    VS_OUTPUT output = (VS_OUTPUT)0;
 
    // Calculate where the vertex ought to be.  This line is equivalent
    // to the transformations in the CelVertexShader.
    float4  v_original = mul(mul(mul(input.Position, World), View), Projection);
 
    // Calculates the normal of the vertex like it ought to be.
    float4 normal = mul(mul(mul(input.Normal, World), View), Projection);
	normal = normalize(normal);
 
    // Take the correct "original" location and translate the vertex a little
    // bit in the direction of the normal to draw a slightly expanded object.
    // Later, we will draw over most of this with the right color, except the expanded
    // part, which will leave the outline that we want.
    output.Position =  v_original + (mul(LineThickness, normal));
 
    return output;
}
 
// The pixel shader for the outline.  It is pretty simple:  draw everything with the
// correct line color.
float4 OutlinePixelShader(VS_OUTPUT input) : COLOR0
{
    return LineColor;
}


//----------------------------- TECHNIQUES --------------------------------

// The entire technique for doing toon shading
technique Toon
{
    // The first pass will go through and draw the back-facing triangles with the outline shader,
    // which will draw a slightly larger version of the model with the outline color.  Later, the
    // model will get drawn normally, and draw over the top most of this, leaving only an outline.
    pass Pass1
    {
        VertexShader = compile vs_2_0 OutlineVertexShader();
        PixelShader = compile ps_2_0 OutlinePixelShader();
        CullMode = CW;
    }
 
    // The second pass will draw the model like normal, but with the cel pixel shader, which will
    // color the model with certain colors, giving us the cel/toon effect that we are looking for.
    pass Pass2
    {
        VertexShader = compile vs_2_0 CelVertexShader();
        PixelShader = compile ps_2_0 CelPixelShader();
        CullMode = CCW;
    }
}